#include "Point3D.h"
#include "Matrix3x3.h"

#include <math.h>


const float Pi = 3.1415926535897932384626433832795f;
const float PiGrad = 0.017453292519943295769236907684886f;


Point3D::Point3D(void){
	Point3D(0.0, 0.0, 0.0);
}

Point3D::Point3D(float x, float y, float z){
	this->x = x;
	this->y = y;
	this->z = z;

	return;
}

Point3D::~Point3D(){}

void Point3D::Set(float x, float y, float z){
	this->x = x;
	this->y = y;
	this->z = z;

	return;
}

void Point3D::Set(Point3D ptArg){
	x = ptArg.x;
	y = ptArg.y;
	z = ptArg.z;

	return;
}

Point3D Point3D::rotate(float fDegX, float fDegY, float fDegZ){
	float CosX, CosY, CosZ;
	float SinX, SinY, SinZ;

	//fDegX = fDegX * PiGrad;
	//fDegY = fDegY * PiGrad;
	//fDegZ = fDegZ * PiGrad;

	CosX = cos(fDegX); SinX = sin(fDegX);
	CosY = cos(fDegY); SinY = sin(fDegY);
	CosZ = cos(fDegZ); SinZ = sin(fDegZ);
	
	Matrix3x3 mxRotation(	CosZ * CosY,		SinZ * CosX + CosZ * SinY * SinX,	SinZ * SinX - CosZ * SinY * CosX,
							-1 * SinZ * CosY,	CosZ * CosX - SinZ * SinY * SinX,	CosZ * SinX + SinZ * SinY * CosX,
							SinY,				-1 * CosY * SinX,					CosY * CosX	);
    

	return mxRotation * (*this);
}

void Point3D::backrotate(Point3D ptTarget, float *fDegX, float *fDegY, float *fDegZ){
	*fDegX = (float)acos((*this & Point3D(1.0f, 0.0f, 0.0f)) / this->norm()) - (float)acos((ptTarget & Point3D(1.0f, 0.0f, 0.0f)) / ptTarget.norm());
	*fDegY = (float)acos((*this & Point3D(0.0f, 1.0f, 0.0f)) / this->norm()) - (float)acos((ptTarget & Point3D(0.0f, 1.0f, 0.0f)) / ptTarget.norm());
	*fDegZ = (float)acos((*this & Point3D(0.0f, 0.0f, 1.0f)) / this->norm()) - (float)acos((ptTarget & Point3D(0.0f, 0.0f, 1.0f)) / ptTarget.norm());

	return;
}

Point3D Point3D::operator -(){
	return Point3D(-x, -y, -z);
}

Point3D Point3D::operator +(Point3D ptArg){
	return Point3D(x + ptArg.x, y + ptArg.y, z + ptArg.z);
}

Point3D Point3D::operator -(Point3D ptArg){
	return Point3D(x - ptArg.x, y - ptArg.y, z - ptArg.z);
}

Point3D Point3D::operator *(float fFac){
	return Point3D(x * fFac, y * fFac, z * fFac);
}

Point3D Point3D::operator /(float fFac){
	return Point3D(x / fFac, y / fFac, z / fFac);
}

Point3D& Point3D::operator+=(Point3D& ptArg){
	x += ptArg.x;
	y += ptArg.y;
	z += ptArg.z;

	return *this;
}

Point3D& Point3D::operator-=(Point3D& ptArg){
	x -= ptArg.x;
	y -= ptArg.y;
	z -= ptArg.z;

	return *this;
}

Point3D& Point3D::operator/=(float fFac){
	x /= fFac;
	y /= fFac;
	z /= fFac;

	return *this;
}

Point3D& Point3D::operator*=(float fFac){
	x *= fFac;
	y *= fFac;
	z *= fFac;

	return *this;
}

Point3D Point3D::operator ^(Point3D ptArg){
	Point3D ptTemp;

	ptTemp.x = y * ptArg.z - z * ptArg.y;
	ptTemp.y = ptArg.x * z - ptArg.z * x;
	ptTemp.z = x * ptArg.y - y * ptArg.x;

	return ptTemp;
}

double Point3D::operator &(Point3D ptArg){
	return (double)(x * ptArg.x + y * ptArg.y + z * ptArg.z);
}

bool Point3D::operator <=(Point3D ptComp)
{
	return (x <= ptComp.x && y <= ptComp.y && z <= ptComp.z);
}

bool Point3D::operator >=(Point3D ptComp)
{
	return (x >= ptComp.x && y >= ptComp.y && z >= ptComp.z);
}

float Point3D::norm(void){
	return sqrt(pow(x, 2) + pow(y, 2) + pow(z, 2));
}
