
#include "Polygon2D.h"

#include "global.h"
#include "Triangle2D.h"
#include "Ray2D.h"
#include "Segment2D.h"


Polygon2D::Polygon2D()
{

}

Polygon2D::Polygon2D(std::vector<Point2D> vVertices)
{
	this->vVertices = vVertices;
}

double Polygon2D::Area()
{
	ENSURE(vVertices.size() >= 3);

	double dArea = 0.0;
	for(unsigned int i = 0; i < vVertices.size() - 1; i++)
	{
		dArea += Triangle2D(Point2D(0.0, 0.0), vVertices[i], vVertices[(i + 1)]).SignedArea();
	}

	return abs(dArea);
}

bool Polygon2D::isConvex()
{
	ENSURE(vVertices.size() >= 3);

	SIGN signFirst = sign(Triangle2D(vVertices[0], vVertices[1], vVertices[2]).SignedArea());
	
	unsigned int uiSize = vVertices.size();
	for(unsigned int i = 1; i < uiSize - 1; i++)
	{
		if(sign(Triangle2D(vVertices[i], vVertices[(i + 1)], vVertices[(i + 2) % uiSize]).SignedArea()) != signFirst)
			return false;
	}

	return true;
}

bool Polygon2D::Contains(Point2D ptTest)
{
	unsigned int uiCountCross = 0;
	for(unsigned int i = 0; i < vVertices.size() - 1; i++)
	{
		if(((vVertices[i].y < vVertices[i + 1].y && vVertices[i].y <= ptTest.y && vVertices[i + 1].y > ptTest.y) && 
			(vVertices[i].y > vVertices[i + 1].y && vVertices[i].y > ptTest.y && vVertices[i + 1].y <= ptTest.y)) &&
			(vVertices[i].x > ptTest.x || vVertices[i + 1].x > ptTest.x)){

			double dScale = (ptTest.y - vVertices[i].y) / (vVertices[i + 1].y - vVertices[i].y);
			double dInterX = (vVertices[i].x + dScale * (vVertices[i + 1].x - vVertices[i].x));
			if (ptTest.x < dInterX)
				uiCountCross++;
		}
	}

	// Return 0/false if uiCountCross is even (point outside polygon) otherwise 1/true
	return !(uiCountCross & 1);
}
